
sets
p periods /p1*p2880/
subp(p) periods for starting point of storage
g generators /g1*g1/
k consumers /k1*k1/
;

Alias(p,pp)


Parameter F(p,p) Demurrage function for consumers;
F(p,pp)=0;

Scalar DL Demurrage length (mins) /120/

*loop to define demurrage function for consumers
loop(p,
if(ord(p)=1,
loop(pp,
if(ord(pp)=1,
F(p,pp)=1;
);
if(ord(pp)>1,
F(p,pp)=max(F(p,pp-1)-(1/DL),0);
);
);
);

if(ord(p)>1,
F(p,pp)=F(p-1,pp-1);
);
);



display F;

parameter FT(pp,p) generator demurrage function;
*Generator demurrage function is transpose of consumers
FT(pp,p)=F(p,pp);
display FT;

Parameters
PI_BAR(p),
DEM(p,k),
MAXUP(k,p),
MAXDOWN(k,p),
CAP(p,g),
MC(g)
;


*input demand data
$call gdxxrw.exe inputs_BC_4.xlsx par=DEM rng='Demand'!a1:t2881
parameter DEM
$GDXIN inputs_BC_4.gdx
$load DEM
$GDXIN
;

display DEM;

*input generation data
$call gdxxrw.exe inputs_BC_4.xlsx par=CAP rng='Generators'!a1:t2881
parameter CAP
$GDXIN inputs_BC_4.gdx
$load CAP
$GDXIN
;

*remove negative generation values
CAP(p,g)$(CAP(p,g)<0)=0;

*marginal cost of generation
MC(g)=0;


*adjust capacity values
scalar FAC /100/
display FAC;
CAP(p,g)=FAC*CAP(p,g)/100;

display MC, CAP;


*set max DR parameters
MAXUP(k,p)=0.1*smax(pp, DEM(pp,k));
MAXDOWN(k,p)=MAXUP(k,p);

*set pay-as-you price
PI_BAR(p)=8.5;
display MAXUP,MAXDOWN, PI_BAR;


*define variables
positive variables
eb(k,p,pp) Enertoks bought by consumers
dr_up(p,k)  DR up
dr_down(p,k) DR down
n(k,p) Pay-as-you-go energy
es(g,p,pp) Enertoks sold be generators

Variables
o;

*define variables
Equations
Obj   Objective function
Max_up(k,p) Maximum DR up
Max_down(k,p) Maximum DR down
DR_bal(k) Demand Repsonse balancing equation
Dem_con(k,p) Consumer demand constraint
Max_CAP(p,g) Maximum capacity constraint
MCC(p) Market Clearing Condition
;

Obj.. o=e=sum((g,p,pp),MC(g)*es(g,p,pp))+sum((k,p),PI_BAR(p)*n(k,p));
DR_bal(k).. sum(p,dr_up(p,k)-dr_down(p,k))=e=0;
Max_up(k,p)..  dr_up(p,k)=l=MAXUP(k,p);
Max_down(k,p)..  dr_down(p,k)=l=MAXDOWN(k,p);
Dem_con(k,p)..   n(k,p)+sum(pp,F(pp,p)*eb(k,pp,p))=e=DEM(p,k)+dr_up(p,k)-dr_down(p,k);
Max_CAP(p,g).. sum(pp,es(g,pp,p))=l=CAP(p,g);
MCC(p).. sum(g,sum(pp,FT(p,pp)*es(g,p,pp)))=e=sum(k,sum(pp,eb(k,p,pp)));

model BlockchainLP /all/;

eb.fx(k,p,pp)$(ord(p) GE ord(pp)+1)=0;
es.fx(g,p,pp)$(ord(pp) GE ord(p)+1)=0;

option solvelink=5;
Option ResLim=100000;
solve BlockchainLP using LP minimizing o ;


*post-processsing
variable sold(p,g);
sold.l(p,g)=sum(pp,es.l(g,p,pp))+1e-10;
display sold.l;

variable gen(p,g);
gen.l(p,g)=sum(pp,es.l(g,pp,p))+1e-10;
display gen.l;


variable bought(p,k);
bought.l(p,k)=sum(pp,eb.l(k,p,pp))+1e-10;
display bought.l;

variable consumed(p,k);
consumed.l(p,k)=sum(pp,eb.l(k,pp,p))+1e-10;
display consumed.l;

variable enertok_price(p,*);
enertok_price.l(p,'dummy1')=MCC.m(p)+1e-10;
display enertok_price.l;

variable diff(p,k);
diff.l(p,k)=consumed.l(p,k)-bought.l(p,k);
display diff.l;

variable diff_g(p,g);
diff_g.l(p,g)=gen.l(p,g)-sold.l(p,g);
display diff.l;



dr_up.l(p,k)=dr_up.l(p,k)+1e-10;
dr_down.l(p,k)=dr_down.l(p,k)+1e-10;

variable con_costs(p,k);
con_costs.l(p,k)=PI_BAR(p)*n.l(k,p)+MCC.m(p)*sum(pp,eb.l(k,p,pp));
display con_costs.l;

variable n2(p,k);
n2.l(p,k)=n.l(k,p)+1e-10;


*output values
execute_unload "results_BC_D4_T2880_F100.gdx"

execute 'gdxxrw.exe results_BC_D4_T2880_F100.gdx O=results_BC_D4_T2880_F100.xls var=sold.l rng=Toks_sold_by_gen!a1'
execute 'gdxxrw.exe results_BC_D4_T2880_F100.gdx O=results_BC_D4_T2880_F100.xls var=gen.l rng=Elec_gen!a1'
execute 'gdxxrw.exe results_BC_D4_T2880_F100.gdx O=results_BC_D4_T2880_F100.xls var=bought.l rng=Toks_bought_by_con!a1'
execute 'gdxxrw.exe results_BC_D4_T2880_F100.gdx O=results_BC_D4_T2880_F100.xls var=diff_g.l rng=Diff_generators!a1'
execute 'gdxxrw.exe results_BC_D4_T2880_F100.gdx O=results_BC_D4_T2880_F100.xls var=consumed.l rng=Energy_consumed!a1'
execute 'gdxxrw.exe results_BC_D4_T2880_F100.gdx O=results_BC_D4_T2880_F100.xls var=diff.l rng=Diff_consumers!a1'
execute 'gdxxrw.exe results_BC_D4_T2880_F100.gdx O=results_BC_D4_T2880_F100.xls var=enertok_price.l rng=Price!a1'
execute 'gdxxrw.exe results_BC_D4_T2880_F100.gdx O=results_BC_D4_T2880_F100.xls var=dr_up.l rng=dr_up!a1'
execute 'gdxxrw.exe results_BC_D4_T2880_F100.gdx O=results_BC_D4_T2880_F100.xls var=dr_down.l rng=dr_down!a1'
execute 'gdxxrw.exe results_BC_D4_T2880_F100.gdx O=results_BC_D4_T2880_F100.xls var=con_costs.l rng=con_costs!a1'
execute 'gdxxrw.exe results_BC_D4_T2880_F100.gdx O=results_BC_D4_T2880_F100.xls var=n2.l rng=Pay_as_you!a1'




